package org.junitdoc.ui.decisiontable;

import java.util.List;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.internal.ui.javaeditor.JavaEditor;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;
import org.junitdoc.JUnitDocPlugin;
import org.junitdoc.core.decisiontable.DecisionTableModel;
import org.junitdoc.core.decisiontable.TestMethod;
import org.junitdoc.core.decisiontable.Type;
import org.junitdoc.core.rewriter.ASTUtils;
import org.junitdoc.core.rewriter.ChangeStateHandler;
import org.junitdoc.core.rewriter.JDTUtils;
import org.junitdoc.core.rewriter.Rewriter;
import org.junitdoc.preferences.PreferenceConstants;
import org.junitdoc.preferences.PreferenceUtils;
import org.junitdoc.ui.Const;
import org.junitdoc.ui.UIMessages;

public class DecisionTableViewer extends TreeViewer {

	private String testTargetMethod;

	private DecisionTableModel model;

	public DecisionTableViewer(final Composite parent) {
		super(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION);

		final Tree tree = getTree();

		GridData gd = new GridData(GridData.FILL_BOTH);
		tree.setLayoutData(gd);
		setAutoExpandLevel(TreeViewer.ALL_LEVELS);

		tree.setHeaderVisible(true);
		tree.setLinesVisible(true);

		setContentProvider(new DecisionTableContentProvider());

		tree.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				int index = tree.getSelectionCount();
				if (index == -1) {
					return;
				}

				JUnitDocPlugin.debugLog("event:" + e);

				Point point = new Point(e.x, e.y);
				TreeItem row = tree.getItem(point);
				if (row == null) {
					return;
				}

				if (row.getBounds(0).contains(point)) {
					// TODO implements operation for add sub tree item.
				}

				for (int columnIndex = 1; columnIndex < tree.getColumnCount(); columnIndex++) {
					if (row.getBounds(columnIndex).contains(point)) {
						if (row.getText(columnIndex) == null) {
							return;
						}
						Object data = row.getData();

						TreeNode treeItem = (TreeNode) data;
						Type data2 = (Type) treeItem.getValue();

						changeStatus(data2, columnIndex);
					}
				}
			}
		});
	}

	public void setName(String testTargetMethodName) {
		this.testTargetMethod = testTargetMethodName;
	}

	public void changeStatus(Type type, int columnIndex) {
		// Index of TestCase start zero, so minus one.
		TestMethod testMethod = model.getTestMethod(columnIndex - 1);

		CompilationUnit ast = (CompilationUnit) type
				.getAnnotationTypeDeclaration().getRoot();

		ICompilationUnit source = JDTUtils.getICompilationUnit(ast);

		Rewriter.execute(source, new ChangeStateHandler(type, testMethod));

		// force refresh here because of affecting rewrote source;
		ast = ASTUtils.createAST(source);
		List<MethodDeclaration> testAllMethods = ASTUtils
				.findAnnotatedMethodDeclarations(ast, Const.TEST_ANNOTATION);

		refresh(testAllMethods);
	}

	public void initialize(CompilationUnit ast) {
		clearAllColumn();

		model = new DecisionTableModel(testTargetMethod);
		model.parse(ast);

		makeColumn(model);

		setInput(model);
	}

	public DecisionTableModel getModel() {
		return model;
	}

	private void makeColumn(DecisionTableModel model) {
		// create First column
		TreeViewerColumn viewerColumn = new TreeViewerColumn(this, SWT.LEFT);
		viewerColumn.setLabelProvider(new HeaderCellLabelProvider());
		TreeColumn column = viewerColumn.getColumn();
		column.setText(UIMessages.getString("DecisionTableViewer.header.text"));
		column.setWidth(200);
		// viewerColumn.setEditingSupport(new MyEditingSupport(this));

		int columnStartPoint = 1;

		for (TestMethod testMethod : model.getTestMethodList()) {

			TreeViewerColumn viewerCol = new TreeViewerColumn(this, SWT.LEFT,
					columnStartPoint + testMethod.getIndex());

			viewerCol.setLabelProvider(new CheckStateCellLabelProvider());
			TreeColumn col = viewerCol.getColumn();
			col.setText(testMethod.getName());
			col.setWidth(50);
			col.setMoveable(true);
			col.addSelectionListener(new TestMethodSelectionListener());

			col.setData(Const.TESTMETHOD_MAPPING_KEY, testMethod);
		}
	}

	/**
	 * A listener class for select method in javaEditor when clicking column
	 * header of decision table.
	 * 
	 */
	public static class TestMethodSelectionListener extends SelectionAdapter {

		public void widgetSelected(SelectionEvent event) {
			JavaEditor javaEditor = JDTUtils.getActiveJavaEditor();

			if (javaEditor != null) {
				TreeColumn selectedColumn = (TreeColumn) event.getSource();
				TestMethod testMethod = (TestMethod) selectedColumn
						.getData(Const.TESTMETHOD_MAPPING_KEY);

				MethodDeclaration md = testMethod.getMethodDeclaration();
				int startPosition = md.getName().getStartPosition();

				javaEditor.selectAndReveal(startPosition, md.getName()
						.getLength());
			}
		}
	}

	private class HeaderCellLabelProvider extends CellLabelProvider {

		@Override
		public void update(ViewerCell cell) {
			TreeNode treeNode = (TreeNode) cell.getElement();

			Type type = (Type) treeNode.getValue();
			String name = type.getName();

			// If this type is "Conditions" or "Actions", set to cell it's
			// notation.
			if (type.getDepth() == 0) {

				if (name.equals(Const.CONDITION_TAG)) {
					cell
							.setText(PreferenceUtils
									.getString(PreferenceConstants.DECISION_TABLE_CONDITION_LABEL));

				} else if (name.equals(Const.ACTION_TAG)) {
					cell
							.setText(PreferenceUtils
									.getString(PreferenceConstants.DECISION_TABLE_ACTION_LABEL));
				}

			} else {
				cell.setText(name);
			}

			// if (type instanceof Condition) {
			// cell.setBackground(parent.getDisplay().getSystemColor(
			// SWT.COLOR_WIDGET_HIGHLIGHT_SHADOW));
			//
			// } else if (type instanceof Action) {
			// cell.setBackground(parent.getDisplay().getSystemColor(
			// SWT.COLOR_WIDGET_LIGHT_SHADOW));
			// }

			Widget item = cell.getItem();

			if (item instanceof TableItem) {
				((TableItem) item).setData(type);
			}
		}
	}

	private class CheckStateCellLabelProvider extends CellLabelProvider {

		@Override
		public void update(ViewerCell cell) {
			TreeNode treeNode = (TreeNode) cell.getElement();
			Type type = (Type) treeNode.getValue();

			// Index of TestCase start from zero, so minus one.
			TestMethod testMethod = model
					.getTestMethod(cell.getColumnIndex() - 1);

			cell.setText(testMethod.getCheckState(type));
		}
	}

	public void refresh(List<MethodDeclaration> methodDeclarations) {
		clearAllColumn();
		model.parseTestMethods(methodDeclarations);
		makeColumn(model);
		refresh();
	}

	private void clearAllColumn() {
		TreeColumn[] columns = getTree().getColumns();

		for (TreeColumn column : columns) {
			column.dispose();
		}
	}

}